/* test_trace.c */

/*
 * Copyright 2007-2016 University of New Hampshire Interoperability Laboratory
 * All rights reserved.
 *
 * This file is part of the UNH EXS implementation of the
 * Consortium Specification: Extended Sockets API (ES-API) Issue 1.0,
 * Document Number C050 published by The Open Group, January 2005,
 * and copyright 2005 by The Open Group.
 *
 * This software is available to you under a choice of one of two
 * licenses.  You may choose to be licensed under the terms of the GNU
 * General Public License (GPL) Version 2, available from the file
 * COPYING.GPL in the main directory of this source tree, or the
 * BSD license, available from the file COPYING.BSD in the main directory
 * of this source tree.
 *
 * This material is based upon work supported by the National Science
 * Foundation under Grants No. OCI-1127228 and DGE-0913620.
 *
 * Any opinions, findings, and conclusions or recommendations
 * expressed in this material are those of the authors and do not
 * necessarily reflect the views of the National Science Foundation.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <trace.h>

int _our_debug_flag = 0;

static void
test_rule(const char *rulestr, struct trace_rule *expected)
{
	struct trace_rule actual;
	char *copy;
	char *end;

	memset(&actual, 0, sizeof(actual));

	copy = strdup(rulestr);
	if (our_parse_rule(copy, &end, &actual) != 0) {
		fprintf(stderr, "our_parse_rule failed for %s at byte %td\n",
				rulestr, end - copy);
		exit(EXIT_FAILURE);
	}
	if (expected->file != NULL && actual.file != NULL
			&& strcmp(expected->file, actual.file) != 0) {
		fprintf(stderr, "incorrect rule for %s: expected file %s got %s\n",
				rulestr, expected->file, actual.file);
		exit(EXIT_FAILURE);
	} else if ((expected->file == NULL && actual.file != NULL)
			|| (expected->file != NULL && actual.file == NULL)) {
		fprintf(stderr, "incorrect rule for %s: expected file %s got %s\n",
				rulestr, expected->file, actual.file);
		exit(EXIT_FAILURE);
	} else if (expected->line_min != actual.line_min
			|| expected->line_max != actual.line_max) {
		fprintf(stderr, "incorrect rule for %s: expected %d-%d got %d-%d\n",
				rulestr, expected->line_min, expected->line_max,
				actual.line_min, actual.line_max);
		exit(EXIT_FAILURE);
	} else if (expected->function != NULL && actual.function != NULL
			&& strcmp(expected->function, actual.function) != 0) {
		fprintf(stderr, "incorrect rule for %s: expected function %s got %s\n",
				rulestr, expected->function, actual.function);
		exit(EXIT_FAILURE);
	} else if ((expected->function == NULL && actual.function != NULL)
			|| (expected->function != NULL && actual.function == NULL)) {
		fprintf(stderr, "incorrect rule for %s: expected file %s got %s\n",
				rulestr, expected->file, actual.file);
		exit(EXIT_FAILURE);
	} else if (expected->format != NULL && actual.format != NULL
			&& strcmp(expected->format, actual.format) != 0) {
		fprintf(stderr, "incorrect rule for %s: expected format %s got %s\n",
				rulestr, expected->format, actual.format);
		exit(EXIT_FAILURE);
	} else if ((expected->format == NULL && actual.format != NULL)
			|| (expected->format != NULL && actual.format == NULL)) {
		fprintf(stderr, "incorrect rule for %s: expected format %s got %s\n",
				rulestr, expected->format, actual.format);
		exit(EXIT_FAILURE);
	}
	free(copy);
	fprintf(stderr, "rule %s passed: %s %d-%d %s \'%s\'\n",
			rulestr, actual.file, actual.line_min, actual.line_max,
			actual.function, actual.format);
	free(actual.file);
	free(actual.function);
	free(actual.format);
}	/* test_rule */

static void
test_rule_fail(char *rulestr, ptrdiff_t expected)
{
	struct trace_rule actual;
	char *copy;
	char *end;
	ptrdiff_t diff;

	copy = strdup(rulestr);
	if (our_parse_rule(copy, &end, &actual) == 0) {
		fprintf(stderr, "our_parse_rule for \"%s\": expected failure got %s %d-%d %s\n",
				rulestr, actual.file, actual.line_min, actual.line_max,
				actual.function);
		exit(EXIT_FAILURE);
	}
	diff = end - copy;
	if (diff != expected) {
		fprintf(stderr, "our_parse_rule for \"%s\": failed at byte %td; expected %td\n",
				rulestr, diff, expected);
		exit(EXIT_FAILURE);
	}
	fprintf(stderr, "rule \"%s\" failed at byte %td as expected\n",
			rulestr, diff);
	free(copy);
}	/* test_range */


#define TEST_RULE(rulestr, infile, inline_min, inline_max, infunction, informat) \
	{ \
		struct trace_rule expected = { \
			.file = (infile), \
			.line_min = (inline_min), \
			.line_max = (inline_max), \
			.function = (infunction), \
			.format = (informat), \
		}; \
		test_rule((rulestr), &expected); \
	}

int
main(void)
{
	TEST_RULE("line 1000", NULL, 1000, 1000, NULL, NULL);
	TEST_RULE("line -1000", NULL, 1, 1000, NULL, NULL);
	TEST_RULE("line 1-1000", NULL, 1, 1000, NULL, NULL);
	TEST_RULE("line 100-1000", NULL, 100, 1000, NULL, NULL);
	TEST_RULE("file exs.c", "exs.c", 0, 0, NULL, NULL);
	TEST_RULE("function exs_init", NULL, 0, 0, "exs_init", NULL);
	TEST_RULE("format hello", NULL, 0, 0, NULL, "hello");
	TEST_RULE("format 'hello world'", NULL, 0, 0, NULL, "hello world");
	TEST_RULE("format \"hello world\"", NULL, 0, 0, NULL, "hello world");
	TEST_RULE("format 'hello\\040world'", NULL, 0, 0, NULL, "hello world");
	TEST_RULE("format 'hello\\40world'", NULL, 0, 0, NULL, "hello world");
	TEST_RULE("format 'hello\\x20world'", NULL, 0, 0, NULL, "hello world");
	TEST_RULE("file exs.c line 500-550 function exs_init format hello",
			"exs.c", 500, 550, "exs_init", "hello");
	TEST_RULE("format 'abc' format 'xyz'", NULL, 0, 0, NULL, "xyz");
	test_rule_fail("", 0);
	test_rule_fail("abc", 0);
	test_rule_fail("-", 0);
	test_rule_fail("10-20", 0);
	test_rule_fail("'def'", 0);
	test_rule_fail("line -", 5);
	test_rule_fail("line --", 6);
	test_rule_fail("line -1-", 6);
	test_rule_fail("line --1", 6);
	test_rule_fail("line 100--1000", 9);
	test_rule_fail("line 1000-100", 5);
	test_rule_fail("format hello\\", 12);
	test_rule_fail("format hello\\x", 12);
	test_rule_fail("format '", 7);
	test_rule_fail("format \"", 7);
	test_rule_fail("format 'hello world", 7);
	test_rule_fail("format \"hello world", 7);
}	/* main */
